"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const node_fs_1 = require("node:fs");
const node_path_1 = require("node:path");
const handlebars_1 = require("handlebars");
function loadTemplate(templateName) {
    const path = (0, node_path_1.join)('templates', templateName);
    return (0, node_fs_1.readFileSync)(path, { encoding: 'utf8' });
}
function buildIconPath(iconStyle, iconType) {
    return `@fortawesome/${iconType}-${iconStyle}-svg-icons`;
}
function generateComponent(icon, outputPath, template, custom = false) {
    const { name, componentName, style, iconType, svgPathData } = icon;
    const context = custom ? { iconName: name, componentName, svgPathData, custom } : { iconName: name, iconPath: buildIconPath(style, iconType), componentName };
    const rendered = template(context);
    if (!(0, node_fs_1.existsSync)(outputPath)) {
        (0, node_fs_1.mkdirSync)(outputPath, { recursive: true });
    }
    const fileName = `${componentName}.tsx`;
    const filePath = (0, node_path_1.join)(outputPath, fileName);
    (0, node_fs_1.writeFileSync)(filePath, rendered, { flag: 'w' });
}
function generateStory(componentName, storiesPath, template) {
    const rendered = template({ componentName });
    if (!(0, node_fs_1.existsSync)(storiesPath)) {
        (0, node_fs_1.mkdirSync)(storiesPath, { recursive: true });
    }
    const fileName = `${componentName}.stories.tsx`;
    const filePath = (0, node_path_1.join)(storiesPath, fileName);
    (0, node_fs_1.writeFileSync)(filePath, rendered, { flag: 'w' });
}
class Generate extends core_1.Command {
    async run() {
        const { flags } = await this.parse(Generate);
        const configFile = flags.config;
        const exportPath = flags.export;
        if (configFile) {
            const configContent = (0, node_fs_1.readFileSync)(configFile, { encoding: 'utf8' });
            const config = JSON.parse(configContent);
            const iconTemplateString = loadTemplate('icon_template.tsx.hbs');
            const iconTemplate = (0, handlebars_1.compile)(iconTemplateString);
            const storyTemplate = (0, handlebars_1.compile)(loadTemplate('story_template.hbs'));
            for (const icon of config.icons) {
                generateStory(icon.componentName, config.storiesPath, storyTemplate);
                generateComponent(icon, config.outputPath, iconTemplate, icon.iconType === 'custom');
            }
            // Adds an index.tsx file in the same directory as the generated components
            // and exports all generated components.
            if (exportPath) {
                // TODO: Maybe find a better name for this template?
                const exportsTemplateString = loadTemplate('index.tsx.hbs');
                const exportsTemplate = (0, handlebars_1.compile)(exportsTemplateString);
                const exportsContext = { iconComponents: config.icons.map(icon => icon.componentName) };
                const rendered = exportsTemplate(exportsContext);
                const filePath = (0, node_path_1.join)(config.outputPath, 'index.tsx');
                (0, node_fs_1.writeFileSync)(filePath, rendered);
            }
        }
        else {
            this.error('Error: No config file specified, please provide one with the --config flag');
        }
    }
}
exports.default = Generate;
Generate.description = 'Generates an icon wrapper component for FontAwesome Icons';
Generate.flags = {
    config: core_1.Flags.string({ char: 'c', description: 'Path to the config file' }),
    export: core_1.Flags.boolean({ char: 'e', description: 'Output an index.ts file in the output directory with all generated files exported' })
    // destructive: Flags.boolean({ char: 'd', description: 'This will delete any existing directories and files in the given paths if they exist before generating the new ones.' })
};
